function single_reaction_results_mvNormal
%-----------------------Pedro Saa 2016-------------------------------------
clc,close all
%%%%%%%%%%%%%%%%%%%%%%%%%% TPI example %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kcat_f   = 4827;										% max turnover in the forward direction (1/s)
kcat_r   = 364;											% max turnover in the reverse direction (1/s)
Km_g3p   = 16e-6;										% g3p disocciation constant (M)
Km_dhap  = 560e-6;										% dhap disocciation constant (M)
RT       = 8.314*298.15/1000;     						% thermodynamic constant
g3p_ref  = 6e-6;										% in vivo concentrations from Brown & Cooper, 1993 (~0.205 uM equilibrium)
dhap_ref = 95e-6;										% idem as before
DGr_std  = -RT*log((kcat_f/Km_g3p)/(kcat_r/Km_dhap));	% Standard Gibbs Free Energy of reaction (kJ/mol) (from Haldane relationship)

% TPI kinetics
tpi = @(g3p,dhap)  (kcat_f*g3p/Km_g3p - kcat_r*dhap/Km_dhap) / (1 + g3p/Km_g3p + dhap/Km_dhap);

% Sampling paremeters
rng('default');             						% For reproducibility
priorRev     = ones(1,3);                           % Prior definitions
priorEnz     = ones(1,3);
reactionFlux = tpi(g3p_ref,dhap_ref);    			% reference flux (M/s)
gibbsEnergy  = DGr_std + RT*log(dhap_ref/g3p_ref);  % Gibbs Free Energy of reaction (kJ/mol)
forwardFlux  = [1,2;2,3;3,1];                       % link matrix (as defined in Saa & Nielsen, 2015)
xdata        = [Km_g3p/g3p_ref,Km_dhap/dhap_ref];   % g3p saturation
N            = 2e3;                                 % number particles
tolerance    = Inf;                                 % tolerance schedule
tol          = 0.15;                                % final tolerance
alpha        = 50;                                  % percentile of alive particles

% Initializing sampled parameters
population(1).kparams = zeros(N,6);
population(1).params  = zeros(N,6);
population(1).rho     = zeros(N,1);
population(1).weights = ones(N,1)/N;

%% I) Prior sampling
tstart  = tic;
popIdx  = 1;
for counter = 1:N
    
    % Sample reversibilities
    revParam  = randg(priorRev);
    randomRev = revParam/sum(revParam);
    
    % Sample enzyme abundances
    enzParam  = randg(priorEnz);
    randomEnzymes = enzParam/sum(enzParam);
    
    % Calulate kinetic parameters (simulate parameters)
    enzymeVect  = [randomEnzymes(forwardFlux(:,1))';randomEnzymes(forwardFlux(:,2))'];
    revTemp     = exp(randomRev*gibbsEnergy/RT).^(sign(reactionFlux));
    revCalIrrev = (1-revTemp).^(-1);
    revCal      = [revCalIrrev';(revTemp.*revCalIrrev)'];
    kparams     = (revCal(:)*reactionFlux).*(enzymeVect(:).^(-1));      
    xsim        = [(kparams(2)*kparams(4) + kparams(2)*kparams(5) + kparams(3)*kparams(5)) / (kparams(1)*(kparams(3) + kparams(4) + kparams(5))),...
                   (kparams(2)*kparams(4) + kparams(2)*kparams(5) + kparams(3)*kparams(5)) / (kparams(6)*(kparams(2) + kparams(3) + kparams(4)))];

    % Accept/Reject particle
    rho = max(abs(xsim-xdata));                                      % Infinite tol
    population(1).kparams(counter,:) = kparams(:)';
    population(1).params(counter,:)  = [randomRev,randomEnzymes];    % Save reversibilities and enzymes individually
    population(1).rho(counter)       = rho;
end
ESS = N;
acceptanceRate = 1;

%% II Sequential Sampling
while (tolerance(popIdx)~=tol)      
    
    % Define tolerance and population statistics
    currTol = max([prctile(population(popIdx).rho,alpha),tol])                                     % Determine tolerance for next step    
	droppedParticles = find(population(popIdx).rho>currTol);
    population(popIdx).weights(droppedParticles)  = [];                                             % Remove particles below the threshold    
    population(popIdx).params(droppedParticles,:) = [];
    population(popIdx).rho(droppedParticles)      = [];
    
    % Re-normalize previous weights
    population(popIdx).weights = population(popIdx).weights/sum(population(popIdx).weights);      	
	
	% Start of next sampling step
    popIdx            = popIdx+1;																  	% Update population and tolerance
    tolerance(popIdx) = currTol;
	Nreplenish        = N-numel(population(popIdx-1).weights);                                  	% Particles to replenish
	Nalive            = N-Nreplenish;																% Alive particles for current iteration
    
    % Extract previous variables
    revPrev = population(popIdx-1).params(:,1:2);                                                   % Extract only independent values
    enzPrev = population(popIdx-1).params(:,4:5);                                                   % Extract only independent values
    
     % Map variables to Euclidean geometry and compute MVN kernel parameters
    Xprevious = [revPrev,enzPrev];
    muX       = population(popIdx-1).weights'*Xprevious;
    diffX     = Xprevious-muX(ones(size(Xprevious,1),1),:);
    diffW     = population(popIdx-1).weights(:,ones(1,size(Xprevious,2))).*diffX;
    covX      = diffW'*diffX/(1-sum(population(popIdx-1).weights.^2));                              % weighted unbiased sample cov
	
    % Sampling loop
	counter    = 1;
	totalCount = 1;
	
    while (counter<=Nreplenish)
        totalCount = totalCount+1;
        
        % Resample from previous population
        particleIdx  = sampleParticle(1:Nalive,1,population(popIdx-1).weights);
        
        % Perform perturbation        
        Xproposal = mvnrnd(Xprevious(particleIdx,:),2*covX);
        if any(Xproposal<0) || (sum(Xproposal(1:2)>1)) || (sum(Xproposal(3:4)>1)); continue; end;                
        randomRevNew      = [Xproposal(1:2),1-sum(Xproposal(1:2))];
        randomEnzymesNew  = [Xproposal(3:4),1-sum(Xproposal(3:4))];
        
        % Calulate kinetic parameters (simulate parameters)
        enzymeVect  = [randomEnzymesNew(forwardFlux(:,1))';randomEnzymesNew(forwardFlux(:,2))'];
        revTemp     = exp(randomRevNew*gibbsEnergy/RT).^(sign(reactionFlux));
        revCalIrrev = (1-revTemp).^(-1);
        revCal      = [revCalIrrev';(revTemp.*revCalIrrev)'];
        kparams     = (revCal(:)*reactionFlux).*(enzymeVect(:).^(-1));      
        xsim        = [(kparams(2)*kparams(4) + kparams(2)*kparams(5) + kparams(3)*kparams(5)) / (kparams(1)*(kparams(3) + kparams(4) + kparams(5))),...
                   (kparams(2)*kparams(4) + kparams(2)*kparams(5) + kparams(3)*kparams(5)) / (kparams(6)*(kparams(2) + kparams(3) + kparams(4)))];
        rhoNew      = max(abs(xsim-xdata));                                  % Calculate discrepancy
        if (rhoNew<=currTol)
			
			% Save model parameters
            population(popIdx).kparams(counter,:) = kparams(:)';
            population(popIdx).params(counter,:)  = [randomRevNew,randomEnzymesNew];               % Save reversibilities and enzymes indidually
            population(popIdx).rho(counter,1)     = rhoNew;
            
			% Compute weight for the accepted particle
            priorProb   = probDirichlet(priorEnz,randomEnzymesNew,1)*probDirichlet(priorRev,randomRevNew,1);
            denomWeight = population(popIdx-1).weights'*(mvnpdf(Xprevious,Xproposal,2*covX));            
			population(popIdx).weights(counter,1) = priorProb/denomWeight;
			
			% Update particle counter
			counter = counter+1;
        end		
    end
    
    % Normalize weights for the replenished particles
	population(popIdx).weights = population(popIdx).weights/sum(population(popIdx).weights);

	% Add previous alive particles to the current population
	population(popIdx).params  = [population(popIdx).params;population(popIdx-1).params];
	population(popIdx).rho     = [population(popIdx).rho;population(popIdx-1).rho];
	population(popIdx).weights = [Nreplenish*population(popIdx).weights;Nalive*population(popIdx-1).weights]/N;       % Normalize weights appropiately
    ESS	= [ESS,1/sum(population(popIdx).weights.^2)];                                                                 % Compute effective sample size
    acceptanceRate = [acceptanceRate,Nreplenish/totalCount];
end
tstop = toc(tstart);
clearvars -except population method tolerance ESS acceptanceRate tstop
save('singleReaction_tpi_Ks_Kp_mvNormal.mat');