% Toy Model: Multinomial likelihood
clc,clear,close all

%% I) SMC-ABC posterior (Dirichlet Kernel method = 1)
rng('default');                                  % for reproducibility
xdata = [1,3,6]*5;                                 % observed data
prior = ones(1,3);
N     = 5e3;
method   = 1;       % (1) MLE; (2) Total Var; (3) Gen Var
tol      = 5;
alpha    = 50;
currTol  = 40;
tolSchedule1 = currTol;
weights  = ones(N,1)/N;
thetaSMC = zeros(N,size(prior,2));
rho      = zeros(N,1);
ESS_smc_Dir1 = N;

% Population I: Prior sampling
disp('Sequential Monte Carlo Sampling')
tstartSMC = tic;
counter = 1;
totalCount = 1;
while (counter<=N)
    
    % Sampling phase
    theta = randg(prior);
    theta = theta/sum(theta);
    
    % Simulate distribution
    xsim   = mnrnd(sum(xdata),theta);
    rhoNew = sum(abs(xsim-xdata));
    if rhoNew<currTol
        thetaSMC(counter,:) = theta;
        rho(counter) = rhoNew;
        counter = counter+1;
    end
    totalCount = totalCount+1;
end
acceptanceRateSMC1 = counter/totalCount;

% Population t+1
Kadaptive1 = [];
while (currTol~=tol)
    
    % Define tolerance
    currTol = max([min([currTol-1,prctile(rho,alpha)]),tol]);
    tolSchedule1 = [tolSchedule1,currTol];
    
    % Prepare population for next sample
    dropedParticles             = find(rho>currTol);
    rho(dropedParticles)        = [];
    weights(dropedParticles)    = [];
    thetaSMC(dropedParticles,:) = [];
    weights = weights/sum(weights);   
    
    % Start of next sampling step
    Nreplenish = N-numel(weights);                                     % Particles to replenish
    Nalive     = N-Nreplenish;
    weightsReplenish = zeros(Nreplenish,1);
    thetaReplenish   = zeros(Nreplenish,size(prior,2));
    rhoReplenish     = zeros(Nreplenish,1);
    
    % Compute kernel parameters
    Kparam = getKernelParam(thetaSMC,method,weights);
    Kadaptive1 = [Kadaptive1,Kparam];
    
    % Sequential loop
    counter = 1;
    thetaReplenish = zeros(Nreplenish,size(prior,2));
    totalCount = 1;
    
    while (counter<=Nreplenish)
        
        % Resample from previous population
        prevIdx = sampleParticle(1:Nalive,1,weights);
        
        % Perform perturbation
        newParticle = randg(Kparam*thetaSMC(prevIdx,:)+1);
        newParticle = newParticle/sum(newParticle);
        
        % Simulate HMM
        xsim = mnrnd(sum(xdata),newParticle);
        rhoNew = sum(abs(xsim-xdata));

        % Check accuracy of the new particle
        if (rhoNew<=currTol)         
            thetaReplenish(counter,:) = newParticle;
            rhoReplenish(counter) = rhoNew;
            
            % Compute weight for the accepted particle
            probPrior = probDirichlet(prior,newParticle,1);
            particleWeight = weights'*probDirichlet(Kparam*thetaSMC+1,newParticle,1);
            weightsReplenish(counter) = probPrior/particleWeight;
                        
            % Update particle counter
            counter = counter+1;
        end
        totalCount = totalCount+1;
    end

    % Normalize weights for the all the particles
    weightsReplenish = weightsReplenish/sum(weightsReplenish);
    weights  = [Nalive*weights;Nreplenish*weightsReplenish]/N;
    rho      = [rho;rhoReplenish];
    thetaSMC = [thetaSMC;thetaReplenish];
    ESS_smc_Dir1  = [ESS_smc_Dir1,1/sum(weights.^2)];
    acceptanceRateSMC1 = [acceptanceRateSMC1;Nreplenish/totalCount];
end
toc(tstartSMC)
disp(['Acceptance rates MLE method: ',mat2str(acceptanceRateSMC1)]);

%% I) SMC-ABC posterior (Dirichlet Kernel method = 2)
clearvars -except acceptanceRateSMC1 ESS_smc_Dir1 tolSchedule1 Kadaptive1
rng('default');                                  % for reproducibility
xdata = [1,3,6]*5;                                 % observed data
prior = ones(1,3);
N     = 5e3;
method   = 2;       % (1) MLE; (2) Total Var; (3) Gen Var
tol      = 5;
alpha    = 50;
currTol  = 40;
tolSchedule2 = currTol;
weights  = ones(N,1)/N;
thetaSMC = zeros(N,size(prior,2));
rho      = zeros(N,1);
ESS_smc_Dir2 = N;

% Population I: Prior sampling
disp('Sequential Monte Carlo Sampling')
tstartSMC = tic;
counter = 1;
totalCount = 1;
while (counter<=N)
    
    % Sampling phase
    theta = randg(prior);
    theta = theta/sum(theta);
    
    % Simulate distribution
    xsim   = mnrnd(sum(xdata),theta);
    rhoNew = sum(abs(xsim-xdata));
    if rhoNew<currTol
        thetaSMC(counter,:) = theta;
        rho(counter) = rhoNew;
        counter = counter+1;
    end
    totalCount = totalCount+1;
end
acceptanceRateSMC2 = counter/totalCount;

% Population t+1
Kadaptive2 = [];
while (currTol~=tol)
    
    % Define tolerance
    currTol = max([min([currTol-1,prctile(rho,alpha)]),tol]);
    tolSchedule2 = [tolSchedule2,currTol];
    
    % Prepare population for next sample
    dropedParticles             = find(rho>currTol);
    rho(dropedParticles)        = [];
    weights(dropedParticles)    = [];
    thetaSMC(dropedParticles,:) = [];
    weights = weights/sum(weights);   
    
    % Start of next sampling step
    Nreplenish = N-numel(weights);                                     % Particles to replenish
    Nalive     = N-Nreplenish;
    weightsReplenish = zeros(Nreplenish,1);
    thetaReplenish   = zeros(Nreplenish,size(prior,2));
    rhoReplenish     = zeros(Nreplenish,1);
    
    % Compute kernel parameters
    Kparam = getKernelParam(thetaSMC,method,weights);
    Kadaptive2 = [Kadaptive2,Kparam];
    
    % Sequential loop
    counter = 1;
    thetaReplenish = zeros(Nreplenish,size(prior,2));
    totalCount = 1;
    
    while (counter<=Nreplenish)
        
        % Resample from previous population
        prevIdx = sampleParticle(1:Nalive,1,weights);
        
        % Perform perturbation
        newParticle = randg(Kparam*thetaSMC(prevIdx,:)+1);
        newParticle = newParticle/sum(newParticle);
        
        % Simulate HMM
        xsim = mnrnd(sum(xdata),newParticle);
        rhoNew = sum(abs(xsim-xdata));

        % Check accuracy of the new particle
        if (rhoNew<=currTol)         
            thetaReplenish(counter,:) = newParticle;
            rhoReplenish(counter) = rhoNew;
            
            % Compute weight for the accepted particle
            probPrior = probDirichlet(prior,newParticle,1);
            particleWeight = weights'*probDirichlet(Kparam*thetaSMC+1,newParticle,1);
            weightsReplenish(counter) = probPrior/particleWeight;
                        
            % Update particle counter
            counter = counter+1;
        end
        totalCount = totalCount+1;
    end

    % Normalize weights for the all the particles
    weightsReplenish = weightsReplenish/sum(weightsReplenish);
    weights  = [Nalive*weights;Nreplenish*weightsReplenish]/N;
    rho      = [rho;rhoReplenish];
    thetaSMC = [thetaSMC;thetaReplenish];
    ESS_smc_Dir2  = [ESS_smc_Dir2,1/sum(weights.^2)];
    acceptanceRateSMC2 = [acceptanceRateSMC2;Nreplenish/totalCount];
end
toc(tstartSMC)
disp(['Acceptance rates Total Var. method: ',mat2str(acceptanceRateSMC2)]);

%% III) SMC-ABC posterior (Dirichlet Kernel method = 3)
clearvars -except acceptanceRateSMC1 ESS_smc_Dir1 tolSchedule1 acceptanceRateSMC2 ESS_smc_Dir2 tolSchedule2 Kadaptive1 Kadaptive2
rng('default');                                  % for reproducibility
xdata = [1,3,6]*5;                                 % observed data
prior = ones(1,3);
N     = 5e3;
method   = 3;       % (1) MLE; (2) Total Var; (3) Gen Var
tol      = 5;
alpha    = 50;
currTol  = 40;
tolSchedule3 = currTol;
weights  = ones(N,1)/N;
thetaSMC = zeros(N,size(prior,2));
rho      = zeros(N,1);
ESS_smc_Dir3 = N;

% Population I: Prior sampling
disp('Sequential Monte Carlo Sampling')
tstartSMC = tic;
counter = 1;
totalCount = 1;
while (counter<=N)
    
    % Sampling phase
    theta = randg(prior);
    theta = theta/sum(theta);
    
    % Simulate distribution
    xsim   = mnrnd(sum(xdata),theta);
    rhoNew = sum(abs(xsim-xdata));
    if rhoNew<currTol
        thetaSMC(counter,:) = theta;
        rho(counter) = rhoNew;
        counter = counter+1;
    end
    totalCount = totalCount+1;
end
acceptanceRateSMC3 = counter/totalCount;

% Population t+1
Kadaptive3 = [];
while (currTol~=tol)
    
    % Define tolerance
    currTol = max([min([currTol-1,prctile(rho,alpha)]),tol]);
    tolSchedule3 = [tolSchedule3,currTol];
    
    % Prepare population for next sample
    dropedParticles             = find(rho>currTol);
    rho(dropedParticles)        = [];
    weights(dropedParticles)    = [];
    thetaSMC(dropedParticles,:) = [];
    weights = weights/sum(weights);   
    
    % Start of next sampling step
    Nreplenish = N-numel(weights);                                     % Particles to replenish
    Nalive     = N-Nreplenish;
    weightsReplenish = zeros(Nreplenish,1);
    thetaReplenish   = zeros(Nreplenish,size(prior,2));
    rhoReplenish     = zeros(Nreplenish,1);
    
    % Compute kernel parameters
    Kparam = getKernelParam(thetaSMC,method,weights);
    Kadaptive3 = [Kadaptive3,Kparam];
    
    % Sequential loop
    counter = 1;
    thetaReplenish = zeros(Nreplenish,size(prior,2));
    totalCount = 1;
    
    while (counter<=Nreplenish)
        
        % Resample from previous population
        prevIdx = sampleParticle(1:Nalive,1,weights);
        
        % Perform perturbation
        newParticle = randg(Kparam*thetaSMC(prevIdx,:)+1);
        newParticle = newParticle/sum(newParticle);
        
        % Simulate HMM
        xsim = mnrnd(sum(xdata),newParticle);
        rhoNew = sum(abs(xsim-xdata));

        % Check accuracy of the new particle
        if (rhoNew<=currTol)         
            thetaReplenish(counter,:) = newParticle;
            rhoReplenish(counter) = rhoNew;
            
            % Compute weight for the accepted particle
            probPrior = probDirichlet(prior,newParticle,1);
            particleWeight = weights'*probDirichlet(Kparam*thetaSMC+1,newParticle,1);
            weightsReplenish(counter) = probPrior/particleWeight;
                        
            % Update particle counter
            counter = counter+1;
        end
        totalCount = totalCount+1;
    end

    % Normalize weights for the all the particles
    weightsReplenish = weightsReplenish/sum(weightsReplenish);
    weights  = [Nalive*weights;Nreplenish*weightsReplenish]/N;
    rho      = [rho;rhoReplenish];
    thetaSMC = [thetaSMC;thetaReplenish];
    ESS_smc_Dir3  = [ESS_smc_Dir3,1/sum(weights.^2)];
    acceptanceRateSMC3 = [acceptanceRateSMC3;Nreplenish/totalCount];
end
toc(tstartSMC)
disp(['Acceptance rates Gen. Var. methd: ',mat2str(acceptanceRateSMC3)]);
clearvars -except acceptanceRateSMC1 ESS_smc_Dir1 tolSchedule1 acceptanceRateSMC2 ESS_smc_Dir2 tolSchedule2 acceptanceRateSMC3 ESS_smc_Dir3 tolSchedule3 Kadaptive1 Kadaptive2 Kadaptive3
save('abc_approximation_probabilistic_models');